// LookoutFrame.h
// History:
//  -11/22/99:
//   John Stovin [stovinj@genrad.com] fixed a problem with the splitter.

#pragma once
#ifndef INC_LOOKOUTFRAME
#define INC_LOOKOUTFRAME

/////////////////////////////////////////////////////////////////////////////
// Primitive Outlookbar sample class - under construction
// NOTE: Win32 Pager control will never give us Outlookbar like looks. And its ugly
//
//
// And this has some well known bugs including the following :
// -Does not support bands
// -Scrolls dont disappear like outlook but merely get disabled
// -known redrawing problems

template <UINT TID>
class COutLookBar: public CViewWindowImpl<COutLookBar> 
{
	typedef CViewWindowImpl<COutLookBar>  baseclass;
	typedef CContainedWindowT< CListViewCtrl, CWinTraitsOR<	WS_CHILD|WS_VISIBLE|WS_CLIPCHILDREN|
															WS_CLIPSIBLINGS|LVS_ICON| LVS_SHAREIMAGELISTS|
															LVS_NOSCROLL|LVS_SINGLESEL, 
															WS_EX_WINDOWEDGE|WS_EX_LEFT> > PagerListView;

public:	
	COutLookBar():	m_List(this,1),
					m_bBottomScroll(false),
					m_bTopScroll(false),
					m_bIconSelected(false),
					m_nItem(-1)
	{}
	
	BEGIN_MSG_MAP(COutLookBar)
		MESSAGE_HANDLER(WM_CREATE, OnCreate)
		MESSAGE_HANDLER(WM_SIZE, OnSize)
		CHAIN_MSG_MAP(baseclass)
	ALT_MSG_MAP(1)
		MESSAGE_HANDLER(WM_NCHITTEST, OnHittest)		
		MESSAGE_HANDLER(WM_LBUTTONDOWN, OnLBtnDown)	
		MESSAGE_HANDLER(WM_LBUTTONUP, OnLBtnUp)	
		MESSAGE_HANDLER(WM_NCPAINT, OnPaint)		
	END_MSG_MAP()
	
	LRESULT OnHittest(UINT, WPARAM, LPARAM lParam, BOOL& bHandled)
	{
		bHandled =FALSE;
		POINT pt;
		pt.x = MOUSE_X(lParam);
		pt.y = MOUSE_Y(lParam);
		m_List.ScreenToClient(&pt);

		DrawItemBevel(1, pt); //raised image
		RECT rcItem;m_List.GetItemRect(0,&rcItem,0);
		
		//enable or disable the bottom scroll
		if(rcItem.top < -3 )	
		{	
			m_bBottomScroll = true;	
		}
		else 
		{
			m_bBottomScroll = false; 
			DrawScrollRects();
		}
		
		//enable or disable the top scroll
		m_List.GetItemRect(m_List.GetItemCount()-1,&rcItem,0);
		RECT rc;m_List.GetViewRect(&rc);
		
		if(rcItem.top + 90> rc.bottom){	m_bTopScroll = true;}
		else 
		{
			m_bTopScroll = false; 
			DrawScrollRects();
		}

		return 0L;
	}

	LRESULT OnLBtnUp(UINT, WPARAM, LPARAM lParam, BOOL& bHandled)
	{
		bHandled = FALSE;
		POINT pt;
		pt.x = MOUSE_X(lParam);
		pt.y = MOUSE_Y(lParam);
		
		if(	(m_bTopScroll && PtInRect(&m_rcScrollTop,pt)) ||
			(m_bBottomScroll && PtInRect(&m_rcScrollBottom,pt))
		 )
		{
			DrawScrollRects();
		}
		return 0L;
	}
	LRESULT OnLBtnDown(UINT, WPARAM, LPARAM lParam, BOOL& bHandled)
	{
		bHandled = FALSE;
		POINT pt;
		pt.x = MOUSE_X(lParam);
		pt.y = MOUSE_Y(lParam);
		
		int nSpacing = HIWORD( m_List.SendMessage(LVM_GETITEMSPACING,(WPARAM)TRUE,0));
		
		RECT rc;

		if(m_bTopScroll && PtInRect(&m_rcScrollTop,pt))//If UP
		{
			int nIndex = m_List.GetTopIndex();
					
			//First make the top item go way up
			m_List.GetItemRect(nIndex,&rc,0);
			int nHt = (rc.bottom - rc.top);
			int top = rc.top - nHt - nSpacing ;
			pt.x = rc.left + 8;
					
			DrawScrollRects(false,true,false);
			
			//now shift all the items underneath 1 step upward
			for(int n=nIndex; n < m_List.GetItemCount() ; n++)
			{
				pt.y = top;	
				m_List.SetItemPosition(n,pt);
				top += nHt + nSpacing + 3;
			}
			
			m_List.RedrawItems(nIndex,m_List.GetItemCount());
			bHandled = TRUE;
		}
		else if(m_bBottomScroll && PtInRect(&m_rcScrollBottom,pt))//If DOWN
		{
			int nIndex = m_List.GetItemCount() - 1;//last item
					
			m_List.GetItemRect(nIndex,&rc,0);
			int nHt = (rc.bottom - rc.top);
			int top = rc.bottom + nSpacing + 3;
			RECT rcTop; m_List.GetItemRect(0,&rcTop,0);
			pt.x = rcTop.left + 8;
					
			DrawScrollRects(false,false,true);
			
			//now shift all the items underneath 1 step downward
			for(int n= nIndex; n >= 0  ; n--)
			{
				pt.y = top;
				m_List.SetItemPosition(n,pt);
				top -= nHt + nSpacing + 3;
			}
			
			m_List.RedrawItems(0,nIndex);
			bHandled = TRUE;
		}
		else 
		{
			bHandled = DrawItemBevel(2, pt);
		}
		return 0L;		
	}

	LRESULT OnPaint(UINT , WPARAM , LPARAM , BOOL& bHandled)
	{
		m_List.Invalidate();
		
		DrawScrollRects(true);
				
		bHandled = TRUE;
		return 0;
	}
	
	LRESULT OnSize(UINT , WPARAM , LPARAM , BOOL& )
	{
		m_List.DefWindowProc();
		RECT rc;GetClientRect(&rc);	m_List.SetWindowPos(0,&rc,SWP_NOZORDER);
		for(int i = 0; i < m_List.GetItemCount(); i++)
		{
			POINT pt;m_List.GetItemPosition(i,&pt);
			pt.x = (rc.right - rc.left)/2 - 15;
			m_List.SetItemPosition(i,pt);
			
			DrawScrollRects();
		}
		return 0L;
	}

	LRESULT OnCreate(UINT, WPARAM , LPARAM , BOOL& )
	{
		m_List.Create(m_hWnd,&CWindow::rcDefault);
		m_ImageList.Create( MAKEINTRESOURCE(TID), 33,36, RGB(0,0,255) );
		m_ImageList.SetOverlayImage(0, 1);
		m_List.SetImageList(m_ImageList.m_hImageList,LVSIL_NORMAL);
	
		SetBkColor();
		SetItemTextBkColor();
		SetItemTextForeColor();
		
		return 0L;
	}

	void InsertItem(int nIndex, LPTSTR pszName, int nImage)
	{
		m_List.InsertItem(nIndex,pszName,nImage);
	}
	
	void SetBkColor(COLORREF clr = GetSysColor(COLOR_APPWORKSPACE) )
	{
		m_List.SetBkColor(clr);
	}
	void SetItemTextBkColor(COLORREF clr = GetSysColor(COLOR_APPWORKSPACE) )
	{
		m_List.SetTextBkColor(clr);
	}
	void SetItemTextForeColor(COLORREF clr = RGB(255,255,255) )
	{
		m_List.SetTextColor(clr);
	}


private:
	BOOL DrawItemBevel(UINT nImage, POINT& pt)
	{
		LV_HITTESTINFO Hti;
		Hti.pt.x = pt.x;
		Hti.pt.y = pt.y;
		BOOL b = FALSE;
		m_List.HitTest(&Hti);
		if(Hti.iItem != -1 &&  ( Hti.flags == LVHT_ONITEMICON || Hti.flags == LVHT_ONITEMLABEL) )  
		{
			HDC hdc = m_List.GetDC();
			RECT rc;
			m_List.RedrawItems(Hti.iItem - 1,Hti.iItem - 1);
			m_List.RedrawItems(Hti.iItem + 1,Hti.iItem + 1);
			m_List.GetItemRect(Hti.iItem,&rc,0);
	
			POINT pt = {rc.left + 10, rc.top};

			m_ImageList.Draw(hdc,nImage,pt,ILD_TRANSPARENT);//pressed look if on the icon
			
			m_nItem = Hti.iItem;
			m_bIconSelected = true;							
			m_List.ReleaseDC(hdc);                   
			b = TRUE;
		}
		else if(Hti.flags == LVHT_NOWHERE)
		{
			m_List.RedrawItems(m_nItem,m_nItem);
			m_bIconSelected = false;
		}
		return b;
	}
 
	//

	void DrawScrollRects(bool bPaintDC= false, bool btopPushed = false, bool bBtmPushed = false)
	{
		RECT rc; m_List.GetClientRect(&rc);
		
		HDC hdc = 0;
		PAINTSTRUCT ps;
		if(!bPaintDC)
		{
			hdc = m_List.GetDC();
		}
		else
		{
			hdc = m_List.BeginPaint(&ps);
		}
		//Top scroll
		::SetRect(&m_rcScrollTop,rc.right-27,rc.top+10,rc.right-10,rc.top+27);
		if(btopPushed)
		{
			::DrawFrameControl(hdc, &m_rcScrollTop, DFC_SCROLL, DFCS_SCROLLUP|DFCS_PUSHED);        
		}
		else
		{
			::DrawFrameControl(hdc, &m_rcScrollTop, DFC_SCROLL, DFCS_SCROLLUP);        
		}

		//Bottom one
		::SetRect(&m_rcScrollBottom,rc.right-27,rc.bottom - 87,rc.right-10,rc.bottom-70);
		if(bBtmPushed)
		{
			::DrawFrameControl(hdc, &m_rcScrollBottom, DFC_SCROLL, DFCS_SCROLLDOWN|DFCS_PUSHED);        
		}
		else
		{
			::DrawFrameControl(hdc, &m_rcScrollBottom, DFC_SCROLL, DFCS_SCROLLDOWN);        
		}

		if(!bPaintDC)
		{
			m_List.ReleaseDC(hdc);hdc = 0;
		}
		else
		{
			m_List.EndPaint(&ps);
		}
	}

private:
	bool			m_bBottomScroll;
	bool			m_bTopScroll;
	RECT			m_rcScrollTop;
	RECT			m_rcScrollBottom;
	int				m_nItem;
	CImageList		m_ImageList;
	PagerListView	m_List;
	bool			m_bIconSelected;
};

/////////////////////////////////////////////////////////////////////////////

class CMyListView: public CViewWindowImpl<CMyListView, CListViewCtrl, CWinTraitsOR<LVS_REPORT |  LVS_NOSORTHEADER | LVS_SINGLESEL, WS_EX_WINDOWEDGE> > 
{
public:	
	typedef CViewWindowImpl<CMyListView, CListViewCtrl, CWinTraitsOR<LVS_REPORT |  LVS_NOSORTHEADER | LVS_SINGLESEL, WS_EX_WINDOWEDGE> >  baseclass;
	BEGIN_MSG_MAP(CMyListView)
		MESSAGE_HANDLER(WM_CREATE, OnCreate)
		CHAIN_MSG_MAP(baseclass)
	END_MSG_MAP()
	
	BOOL AddColumn(LPCTSTR strItem,int nItem,UINT nImageID = -1,int nSubItem = -1,
			int nMask = LVCF_FMT | LVCF_WIDTH | LVCF_TEXT | LVCF_IMAGE|LVCF_SUBITEM,
			int nFmt = LVCFMT_LEFT| LVCFMT_IMAGE)
	{
		ATLASSERT(::IsWindow(m_hWnd));
		LV_COLUMN lvc;
		lvc.mask = nMask;
		lvc.fmt = nFmt;
		if(nImageID != -1)
			lvc.iImage = nImageID;

		lvc.pszText = (LPTSTR)strItem;
		lvc.cx = GetStringWidth(lvc.pszText) + 15;
		if(nMask & LVCF_SUBITEM)
		{
			if(nSubItem != -1)
				lvc.iSubItem = nSubItem;
			else
				lvc.iSubItem = nItem;
		}
		return InsertColumn(nItem, &lvc);
	}

	LRESULT OnCreate(UINT , WPARAM , LPARAM , BOOL& )
	{
		DefWindowProc();
		SetExtendedListViewStyle(LVS_EX_FULLROWSELECT);

		m_ImageList.Create( MAKEINTRESOURCE(IDB_HEADER1), 12,17, RGB(0,0,255) );
				
		//TODO Add your data
		AddColumn(_T("   "), 0,0);
		AddColumn(_T("   "), 1,1);
		AddColumn(_T("   "), 2,2);
		AddColumn(_T("   "), 3,3);

		AddColumn(_T("From                     "), 4);
		AddColumn(_T("Subject                              "), 5);

		AddItem(0,0,_T(""));
		SetItemText(0, 4,_T("Hi there ! How are you - its been a long time..."));
		
		AddItem(1,0,_T(""));
		SetItemText(1, 4,_T("Hi there ! How are you - its been a long time..."));
				
		HWND hWndHeader = GetHeader();
		::SetWindowLong(hWndHeader,GWL_STYLE,::GetWindowLong(hWndHeader, GWL_STYLE)|HDS_BUTTONS);
		Header_SetImageList(hWndHeader,m_ImageList.m_hImageList);
		
		return 0L;
	}
	CImageList  m_ImageList;
};

/////////////////////////////////////////////////////////////////////////////
// Right Pane of the SplitterView 

class CRightViewChild : public CViewWindowImpl<CRightViewChild>
{
public:
	CRightViewChild():m_ListViewTop(this){}	
public:
	BEGIN_MSG_MAP(CRightViewChild)
		MESSAGE_HANDLER(WM_CREATE, OnCreate)
		MESSAGE_HANDLER(WM_SIZE, OnSize)		
		CHAIN_MSG_MAP(CViewWindowImpl<CRightViewChild>)
	END_MSG_MAP()
	
	LRESULT OnCreate(UINT , WPARAM , LPARAM , BOOL& )
	{
		m_ListViewBottom.Create(m_hWnd,CWindow::rcDefault);
		m_ListViewTop.Create(m_hWnd,CWindow::rcDefault);
		return 0L;
	}
	LRESULT GetSiblingWnd(CWindow& wndSibling)
	{
		if(m_ListViewBottom.m_hWnd)
		{
			wndSibling = m_ListViewBottom.m_hWnd;
			return 0;
		}
		return -1;
	}
	LRESULT OnSize(UINT , WPARAM , LPARAM , BOOL& )
	{
		DefWindowProc();
	
		RECT rc; GetClientRect(&rc);
		
		m_ListViewTop.RepositionSplitters(&rc);
		return 0L;
	}

public:
	typedef CSplitterImpl< CRightViewChild, CMyListView,false> CSplitterView;
	
	CSplitterView			m_ListViewTop;
	CMyListView				m_ListViewBottom;
};

/////////////////////////////////////////////////////////////////////////////

class CRightView: public CViewWindowImpl<CRightView>
{
public:
	CRightView():m_Infobar(L"Attila is way cool !",IDB_BITMAP2){}	
	
	BEGIN_MSG_MAP(CRightViewChild)
		MESSAGE_HANDLER(WM_CREATE, OnCreate)
		MESSAGE_HANDLER(WM_SIZE, OnSize)		
		CHAIN_MSG_MAP(CViewWindowImpl<CRightView>)
	END_MSG_MAP()
	LRESULT OnCreate(UINT , WPARAM , LPARAM , BOOL& )
	{
		m_Infobar.Create(m_hWnd,CWindow::rcDefault);
		m_Child.Create(m_hWnd,CWindow::rcDefault);
		return 0L;
	}
	LRESULT OnSize(UINT , WPARAM , LPARAM , BOOL& )
	{
		DefWindowProc();
	
		RECT rc; GetClientRect(&rc);
		m_Infobar.SetWindowPos(0,rc.left, rc.top, rc.right - rc.left, rc.top+ 50,SWP_NOZORDER);	
		
		rc.top += 50;
		m_Child.SetWindowPos(0,&rc,SWP_NOZORDER);

		return 0L;
	}
	CRightViewChild			m_Child;
private:
	CInfoBar				m_Infobar;
	
};

/////////////////////////////////////////////////////////////////////////////
// Left Pane of the SplitterView 

class CMyView: public CViewWindowImpl<CMyView>
{
public:
	typedef CViewWindowImpl<CMyView> baseClass;
	
	CMyView():m_LeftView(this,(float)0.13)
	{
	}	

public:
	BEGIN_MSG_MAP(CMyView)
		MESSAGE_HANDLER(WM_CREATE, OnCreate)
		MESSAGE_HANDLER(WM_SIZE, OnSize)		
		CHAIN_MSG_MAP(baseClass)
	END_MSG_MAP()

	LRESULT OnCreate(UINT , WPARAM , LPARAM , BOOL& )
	{
		m_RightView.Create(m_hWnd,CWindow::rcDefault);
		m_LeftView.Create(m_hWnd,CWindow::rcDefault);
	
		m_LeftView.GetContainedView()->InsertItem(0,_T("Tasks"),0);
		m_LeftView.GetContainedView()->InsertItem(1,_T("Calendar"),0);
		m_LeftView.GetContainedView()->InsertItem(2,_T("Notes"),0);
		m_LeftView.GetContainedView()->InsertItem(3,_T("Meetings"),0);
		m_LeftView.GetContainedView()->InsertItem(4,_T("Cafeteria Interviews"),0);
		m_LeftView.GetContainedView()->InsertItem(5,_T("To Decaf Zone"),0);
		m_LeftView.GetContainedView()->InsertItem(6,_T("Zen"),0);
		m_LeftView.GetContainedView()->InsertItem(7,_T("Staring"),0);
		m_LeftView.GetContainedView()->InsertItem(8,_T("Rebooting"),0);
		m_LeftView.GetContainedView()->InsertItem(9,_T("Salvation"),0);
		m_LeftView.GetContainedView()->InsertItem(10,_T("Nirvana"),0);
		m_LeftView.GetContainedView()->InsertItem(11,_T("Arppegio"),0);
		m_LeftView.GetContainedView()->InsertItem(12,_T("Dreaming"),0);
	
		return 0L;
	}
	
	LRESULT OnSize(UINT , WPARAM , LPARAM , BOOL& )
	{
		DefWindowProc();
	
		RECT rc; GetClientRect(&rc);
		
		m_LeftView.RepositionSplitters(&rc);
		return 0L;
	}
	LRESULT GetSiblingWnd(CWindow& wndSibling)
	{
		if(m_RightView.m_hWnd)
		{
			wndSibling = m_RightView.m_hWnd;
			return 0;
		}
		return -1;
	}
	
	typedef CSplitterImpl< CMyView, COutLookBar<IDB_BITMAP1> > CSplitterView;

private:
	CSplitterView			m_LeftView;
	CRightView				m_RightView;

};

/////////////////////////////////////////////////////////////////////////////
// App Frame

class CLookoutFrame: public CFrameWindowImpl<CLookoutFrame>
{
public:
	typedef CFrameWindowImpl<CLookoutFrame> baseClass;
	

	BEGIN_MSG_MAP(CLookoutFrame)
		MESSAGE_HANDLER(WM_CREATE, OnCreate)
		MESSAGE_HANDLER(WM_SETFOCUS, OnSetFocus)
		MESSAGE_HANDLER(WM_SIZE, OnSize)		
		COMMAND_ID_HANDLER(ID_APP_EXIT, OnAppExit) 
		COMMAND_ID_HANDLER(ID_APP_ABOUT, OnAbout) 
		CHAIN_MSG_MAP(baseClass)
	END_MSG_MAP()

	LRESULT OnCreate(UINT , WPARAM , LPARAM, BOOL&)
	{
		//Create the Statusbar
		m_wndStatusBar.Create(m_hWnd,CWindow::rcDefault);
		
		//Create the View
		m_View.Create(m_hWnd,CWindow::rcDefault);

		//Fill an array of Toolbar buttons
		TBBUTTON tbButtons[] = 
		{
			{ 0,	ID_FILE_NEW,						TBSTATE_ENABLED				,TBSTYLE_BUTTON, 0L, 0},
			{ 1,	ID_FILE_OPEN,						TBSTATE_ENABLED				,TBSTYLE_BUTTON, 0L, 0},
			{ 2,	ID_FILE_SAVE,						TBSTATE_ENABLED				,TBSTYLE_BUTTON, 0L, 0},
			{ 3,	ID_EDIT_CUT,						TBSTATE_ENABLED				,TBSTYLE_BUTTON, 0L, 0},
			{ 4,	ID_EDIT_COPY,						TBSTATE_ENABLED				,TBSTYLE_BUTTON, 0L, 0},
			{ 5,	ID_EDIT_PASTE,						TBSTATE_ENABLED				,TBSTYLE_BUTTON, 0L, 0},
			{ 6,	ID_APP_ABOUT,						TBSTATE_ENABLED				,TBSTYLE_BUTTON, 0L, 0}
		};

		//Create the Toolbar
		//m_wndToolBar.Create(m_hWnd,CWindow::rcDefault,tbButtons,7,IDR_MAINFRAME);
		//Create the Toolpanel
		m_wndToolBar.Create(m_hWnd,CWindow::rcDefault,tbButtons,7,IDR_MAINFRAME,IDR_MAINFRAME);
		
		return 0L;
	}

	LRESULT OnSetFocus(UINT , WPARAM , LPARAM , BOOL& )
	{
		m_View.SetFocus();
		return 0L;
	}

	LRESULT OnSize(UINT, WPARAM wParam, LPARAM , BOOL& bHandled)
	{
		if(wParam == SIZE_MINIMIZED) //very imp - this needs to be in evry top level Frame window's onsize handler
			return 0L;

		bHandled = ResizeElements();
		return 0L;
	}
	LRESULT OnAppExit(WORD,WORD ,HWND , BOOL& )
	{	
		DestroyWindow(); //Will make the Frame call PostQuitMessage
		return 0L;
	}
	LRESULT OnAbout(WORD, WORD , HWND , BOOL& )
	{
		CSimpleDialog<IDD_ABOUT> dlg;
		dlg.DoModal();
		return 0L;
	}
	
public://virtuals
	BOOL ResizeElements()
	{
		m_wndStatusBar.SendMessage(WM_SIZE);
		RECT rc,rcToolPanel,rcStatus;
		GetClientRect(&rc);
		
		GetClientRect(&rc);
		m_wndToolBar.SetWindowPos(NULL,rc.left, rc.top, rc.right - rc.left,60,SWP_NOZORDER | SWP_NOACTIVATE);
				
		m_wndToolBar.GetWindowRect(&rcToolPanel);
		rc.top +=  (rcToolPanel.bottom - rcToolPanel.top);

		m_wndStatusBar.GetWindowRect(&rcStatus);
		rc.bottom -= (rcStatus.bottom - rcStatus.top);
		
		m_View.SetWindowPos(NULL, rc.left, rc.top,rc.right - rc.left, rc.bottom - rc.top,
							SWP_NOZORDER | SWP_NOACTIVATE);

		return TRUE;
	}
	
private:
	CMyView				m_View;
	CCoolToolPanel		m_wndToolBar;
	CStatusBar			m_wndStatusBar;	
};

#endif  // INC_LOOKOUTFRAME
